#ifndef __Options_h__
#define __Options_h__

#include "IO.h"
#include "Sort.h"

typedef struct opts {

    int argc ;
    char **argv ;
  
    /* General options */
    char *matrix ;  /* name of the matrix */
    int P ;         /* number of processors */
    double eps ;    /* allowed load imbalance */
  
    /* Split strategy options */
    enum {Constant, Increase, Decrease} LoadbalanceStrategy ; 
    enum {AdjustNo, AdjustYes} LoadbalanceAdjust; 
    enum {Alternate, LocalBest, Hybrid, LocalRatio,
          OneDimRow, OneDimCol, FineGrain} SplitStrategy ;
    enum {Row, Col, Ratio} Alternate_FirstDirection ; 
    enum {Simple, KLFM} SplitMethod;
  
    /* Matrix options */
    enum {EqVecNo, EqVecYes} SquareMatrix_DistributeVectorsEqual ;
    enum {DumNo, DumYes} SquareMatrix_DistributeVectorsEqual_AddDummies ;
    enum {SingleEntNo, SingleEntYes} SymmetricMatrix_UseSingleEntry ;
    enum {ETypeLower, ETypeRandom} SymmetricMatrix_SingleEntryType ;
  
    /* Coarsening options */
    long Coarsening_NrVertices ;
    long Coarsening_MaxNrVtxInMatch ;
    double Coarsening_StopRatio ;
    double Coarsening_VtxMaxFractionOfWeight ; 
    enum {Random, Inprod} Coarsening_MatchingStrategy ;
    enum {DecreasingWgt, IncreasingWgt, DecreasingDegree, IncreasingDegree,
          NaturalOrder, RandomOrder} Coarsening_InprodMatchingOrder ;
    enum {Ip, IpFine} Coarsening_FineMatchingStrategy ;
    enum {NoNetScaling, Linear} Coarsening_NetScaling ;
    enum {NoIpScaling, Cos, Min, Max, Jaccard} Coarsening_InprodScaling ;
    enum {MatchIdNo, MatchIdYes} Coarsening_MatchIdenticalFirst ;
  
    /* Initial partitioning options */
    long KLFM_InitPart_NrRestarts ;
    long KLFM_InitPart_MaxNrLoops ;
    long KLFM_InitPart_MaxNrNoGainMoves ;
  
    /* Refinement options */
    long KLFM_Refine_MaxNrLoops ;
    long KLFM_Refine_MaxNrNoGainMoves ;
  
    /* Vector partitioning options */
    enum {VecIncrease, VecDecrease, VecRandom} VectorPartition_Step3 ;
    long VectorPartition_MaxNrLoops ;
    long VectorPartition_MaxNrGreedyImproves ;
  
    long Seed ;
  
} Opts ;
  
    /* Function declarations for Options.c */
    void GetParameters(struct opts *pOptions, int argc, char **argv) ;
    void SetDefaults(struct opts *pOptions) ;
    void SetOption(struct opts *pOptions, char *option, char *value) ;
    void PrintHelp(int argc, char **argv) ;
  
#endif /* __Options_h__ */
