#include "Graph.h"
#include "Match.h"

struct opts Options ;

int main(int argc, char **argv) {

    struct biparthypergraph HG ;
    struct contraction C ;

    long i, j, n, t, v, level, *Visited, *Inprod, *NetStart, *NetEnd ;
    int *Matched ;

    printf("Test FindMatchInprod_FineGrain: ");
    n= 12 ; /* must be >= 2 */

    /* Fine-grain hypergraph, corresponding to n by n diagonal matrix
       with extra entry (n-2,n-1)  */
    HG.NrVertices = n+1 ;
    HG.NrNets = 2*n ;
    HG.NrPins = 2*n + 2 ;

    HG.V = (struct vertex *) malloc(HG.NrVertices * sizeof(struct vertex)) ;
    HG.N = (struct net *) malloc(HG.NrNets * sizeof(struct net)) ;
    HG.VtxAdjncy = (long *) malloc(HG.NrPins * sizeof(long)) ;
    HG.NetAdjncy = (long *) malloc(HG.NrPins * sizeof(long)) ;
    Matched = (int *) malloc(HG.NrVertices * sizeof(int)) ;
    Inprod =  (long *) malloc(HG.NrVertices * sizeof(long)) ;
    Visited =  (long *) malloc(HG.NrVertices * sizeof(long)) ;
    NetStart =  (long *) malloc(HG.NrNets * sizeof(long)) ;
    NetEnd =  (long *) malloc(HG.NrNets * sizeof(long)) ;


    if ( HG.V == NULL || HG.N == NULL || 
         HG.VtxAdjncy == NULL || HG.NetAdjncy == NULL || Matched == NULL ||
         Inprod == NULL || Visited == NULL || NetStart == NULL || NetEnd == NULL )
        Abort("test_FindMatchInprod_FineGrain(): Not enough memory") ;

    /* Contraction, at initial stage with no vertices matched yet */
    C.NrMatches = 0 ;
    C.MaxNrVertices = 2 ; /* pairwise matching */
    C.MaxVtxWgt = n*n ;     /* no limit on weight */
    C.Match = (long *) malloc(HG.NrVertices * sizeof(long)) ;
    C.Start = (long *) malloc((HG.NrVertices+1) * sizeof(long)) ;
    if ( C.Match == NULL ||  C.Start == NULL )
        Abort("test_FindMatchInprod_FineGrain(): Not enough memory") ;
    C.Start[0] = 0;

    /* Initialise vertices */
    for (t=0; t<HG.NrVertices ; t++){
        HG.V[t].vtxwgt =  1;
        HG.V[t].iStart = 2*t ; 
        HG.V[t].iEnd = 2*(t+1) ;
        Matched[t] =  FALSE ;
        Inprod[t] = 0 ;

    }

    /* Initialise nets 0 .. 2n-5 alternatingly */
    for (t=0; t<2*n-4; t++){
        HG.N[t].iStartP0 = t ;
        HG.N[t].iStartP1 = t+1 ;
    }    
    /* net 2n-4 (a row-net with 2 pins) */
    HG.N[t].iStartP0 = t ;
    HG.N[t].iStartP1 = t+2 ;
    t++ ;
    
    /* net 2n-3 (a column-net with 1 pin) */
    HG.N[t].iStartP0 = t+1 ;
    HG.N[t].iStartP1 = t+2 ;
    t++ ;
    
    /* net 2n-2 (a row-net with 1 pin) */
    HG.N[t].iStartP0 = t+1 ;
    HG.N[t].iStartP1 = t+2 ;
    t++ ;
    
    /* net 2n-1 (a column-net with 2 pins) */
    HG.N[t].iStartP0 = t+1 ;
    HG.N[t].iStartP1 = t+3 ;

    for (t=0; t<HG.NrNets; t++)
        HG.N[t].iEnd = HG.N[t].iStartP1 ;
 
    /* Initialise each vertex adjacency list with 2 pins */
    for (j= 0; j<n; j++){
        HG.VtxAdjncy[2*j] = 2*j ;
        HG.VtxAdjncy[2*j+1] = 2*j+1 ;
    }
    HG.VtxAdjncy[2*n] = 2*n-4 ;
    HG.VtxAdjncy[2*n+1] = 2*n-1 ;

    /* Initialise each net adjacency list with 1 or 2 pins */
    for (i=0; i<2*n-4; i++)
        HG.NetAdjncy[i] = i/2 ;
    HG.NetAdjncy[2*n-4] = n-2 ;
    HG.NetAdjncy[2*n-3] = n ;
    HG.NetAdjncy[2*n-2] = n-2 ;
    HG.NetAdjncy[2*n-1] = n-1 ;
    HG.NetAdjncy[2*n] = n-1 ;
    HG.NetAdjncy[2*n+1] = n ;

    v = n-2 ;
    level = 0 ;
    FindMatchInprod_FineGrain(&HG, &C, v, Matched, Visited, Inprod,
                              NetStart, NetEnd, level) ;

    /* Check hypergraph dimensions */
    if (HG.NrVertices != n+1 ||
        HG.NrNets != 2*n ||
        HG.NrPins != 2*n + 2) {

        printf("Error\n") ;
        exit(1);
    }

    /* Check Matched, Inprod arrays */
    for (j=0; j<HG.NrVertices; j++){
        if ( (j < n-2 && Matched[j]) ||
             Inprod[j] != 0 ) {
            printf("Error\n") ;
            exit(1);
        }
    }
    if ( (Matched[n-2] == FALSE)  ||
         Matched[n-1] ||
         (Matched[n] == FALSE)  ) {
        printf("Error\n") ;
        exit(1);
    }

    /* Check number of matches */
    if ( C.NrMatches != 1 ||
         C.MaxNrVertices != 2 ||
         C.MaxVtxWgt != n*n ||
         C.Start[0] != 0 ||
         C.Start[1] != 2 ) {
        printf("Error\n") ;
        exit(1);
    }

    /* Check matches in contraction */
    for (t= C.Start[0]; t < C.Start[1]; t++){
        j = C.Match[t] ;
        if (j < n-2 || j == n-1 || Matched[j] == FALSE ) {
            printf("Error\n") ;
            exit(1);
        }
    }

    /* Reset all matched vertices */
    for (t= C.Start[0]; t < C.Start[1]; t++){
        j = C.Match[t] ;
        Matched[j] = FALSE ; 
    }

    /* Check reset Matched array */
    for (j=0; j<HG.NrVertices; j++){
        if ( Matched[j] )  {
            printf("Error\n") ;
            exit(1);
        }
    }

    printf("OK\n") ;
    exit(0);

} /* end main */
